#include <gdk/gdkkeysyms.h>
#include <gtkmm/accelgroup.h>
#include <gtkmm/button.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/label.h>
#include <gtkmm/box.h>

#include "sessionselection.h"
#include <bcore/t_string.h>
#include <bcore/command/command.h>
#include <gtkmm/main.h>

namespace btg
{
   namespace UI
   {
      namespace gui
      {
         using namespace std;
         using namespace btg::core;

         sessionSelectionDialog::sessionSelectionDialog(t_longList const& _sessionlist, bool const _disableSelection)
            : sessionlist(_sessionlist),
              disableSelection_(_disableSelection),
              selected(false),
              session(Command::INVALID_SESSION),
              cbt(0)
         {
            Gtk::Button *cancelbutton = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-cancel")));
            Gtk::Button *okbutton     = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-ok")));
            Gtk::Label *label;
            if (!disableSelection_)
               {
                  label = Gtk::manage(new class Gtk::Label("Select session"));
               }
            else
               {
                  label = Gtk::manage(new class Gtk::Label("Available sessions"));
               }

            cbt                   = Gtk::manage(new class Gtk::ComboBoxText());
            Gtk::VBox *vbox       = Gtk::manage(new class Gtk::VBox(false, 0));

            cancelbutton->set_flags(Gtk::CAN_FOCUS);
            cancelbutton->set_relief(Gtk::RELIEF_NORMAL);
            okbutton->set_flags(Gtk::CAN_FOCUS);
            okbutton->set_relief(Gtk::RELIEF_NORMAL);
            get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
            label->set_alignment(0.5,0.5);
            label->set_padding(0,0);
            label->set_justify(Gtk::JUSTIFY_LEFT);
            label->set_line_wrap(false);
            label->set_use_markup(false);
            label->set_selectable(false);

            vbox->pack_start(*label, Gtk::PACK_SHRINK, 0);
            vbox->pack_start(*cbt);
            get_vbox()->set_homogeneous(false);
            get_vbox()->set_spacing(0);
            get_vbox()->pack_start(*vbox);

            if (!disableSelection_)
               {
                  set_title("Select session");
               }
            else
               {
                  set_title("Available sessions");
               }

            set_modal(true);
            property_window_position().set_value(Gtk::WIN_POS_NONE);
            set_resizable(true);
            property_destroy_with_parent().set_value(false);
            set_has_separator(true);
            add_action_widget(*cancelbutton, -6);
            add_action_widget(*okbutton, -5);

            /// Fill the combobox with session ids.
            t_longListCI sessionIter;

            for (sessionIter = _sessionlist.begin();
                 sessionIter != _sessionlist.end();
                 sessionIter++)
               {
                  cbt->append_text(convertToString<t_long>(*sessionIter));
               }

            // Select the first session id.
            cbt->set_active(0);

            if (!disableSelection_)
               {
                  cancelbutton->show();
               }

            okbutton->show();
            label->show();
            cbt->show();
            vbox->show();
            show();

            // Connect buttons to handlers.
            okbutton->signal_clicked().connect(sigc::mem_fun(*this, &sessionSelectionDialog::on_ok_clicked));
            if (!disableSelection_)
               {
                  cancelbutton->signal_clicked().connect(sigc::mem_fun(*this, &sessionSelectionDialog::on_cancel_clicked));
               }
         }

         void sessionSelectionDialog::on_ok_clicked()
         {
            string active = cbt->get_active_text();
            if (active != "")
               {
                  this->selected = true;
                  this->session  = convertStringTo<t_long>(active);
               }
            else
               {
                  this->selected = false;
               }

            this->hide();
         }

         void sessionSelectionDialog::on_cancel_clicked()
         {
            this->selected = false;
            this->hide();
         }

         bool sessionSelectionDialog::getSelectedSession(t_long & _session)
         {
            if (this->selected)
               {
                  _session = this->session;
               }
            else
               {
                  _session = Command::INVALID_SESSION;
               }

            return this->selected;
         }

         sessionSelectionDialog::~sessionSelectionDialog()
         {
         }

      } // namespace gui
   } // namespace UI
} // namespace btg
